/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.farm.type;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import nl.cloudfarming.client.farm.model.Farm;
import nl.cloudfarming.client.farm.model.FarmManager;
import org.openide.cookies.SaveCookie;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.MultiDataObject;
import org.openide.loaders.MultiFileLoader;
import org.openide.nodes.Node;
import org.openide.util.Lookup;
import org.openide.util.lookup.AbstractLookup;
import org.openide.util.lookup.InstanceContent;
import org.openide.util.lookup.ProxyLookup;

public class FarmDataObject extends MultiDataObject implements PropertyChangeListener{

    private InstanceContent ic;
    private Lookup intern;
    private FarmManager farmManager;
    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.farm");    
    
    public FarmDataObject(FileObject pf, MultiFileLoader loader) throws DataObjectExistsException, IOException {
        super(pf, loader);
        registerEditor("application/x-agrosense-farm", true);
        
        this.ic = new InstanceContent();
        this.intern = new AbstractLookup(this.ic);

        farmManager = Lookup.getDefault().lookup(FarmManager.class);
        assert farmManager != null : "No FarmManager class found in the default lookup, unable to load the DataObject";        
    }

    @Override
    protected int associateLookup() {
        return 1;
    }


    @Override
    protected Node createNodeDelegate() {
        Farm farm = farmManager.load(getPrimaryFile());
        assert farm != null;
        LOGGER.log(Level.FINE, "FarmDataObject loaded {0}", System.identityHashCode(farm));
        ic.add(farm);
        farm.addPropertyChangeListener(this);
        
        LOGGER.log(Level.FINE, "Creating new node delegate for FarmDataObject");           
        
        return super.createNodeDelegate();
    }
    
    @Override
    public void propertyChange(PropertyChangeEvent arg0) {
        SaveCookie saveCookie =
                this.getLookup().
                lookup(SaveCookie.class);
        if (null == saveCookie) {
            this.ic.add(new FieldSaveCapability());
        }
    }

    @Override
    public Lookup getLookup() {
        return new ProxyLookup(super.getLookup(), intern);
    }

    private class FieldSaveCapability implements SaveCookie {

        @Override
        public void save() throws IOException {
            FileObject fo = FarmDataObject.this.getPrimaryFile();
            Farm farm = FarmDataObject.this.getLookup().
                    lookup(Farm.class);
            FarmDataObject.this.farmManager.save(farm, fo);
            FarmDataObject.this.ic.remove(FieldSaveCapability.this);
        }
    }    
}
